// Ecisubs.cpp: implementation of the CEcisubs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Ecisubs.h"
#include "comsubs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEcisubs::CEcisubs()
{

}

CEcisubs::~CEcisubs()
{

}

////////////////////////////////////////////////////////////////
// Routine to execute a CICS transaction
////////////////////////////////////////////////////////////////

int CEcisubs::executeCICSTran(const char* tran, 
							   long casize, 
							   void *dataArea, 
							   const char* cics_system,
							   const char* cics_user,
							   const char* cics_pw,
							   CString& cics_message,
							   CString& cics_abcode)

{
	// Trace point for debugging
	ISAPITRACE("Entering executeCICSTran\n");

	if (casize > 0)
	{
		// Make sure we don't have a null pointer
		ISAPIASSERT(dataArea != NULL);
	}

	// Make sure the COMMAREA size is >= 0
	ISAPIASSERT(casize >= 0);

	int RC = Ccl::noError;

	// Build a connection object
	CclConn connection(cics_system, cics_user, cics_pw);

	// Build a buffer object
	CclBuf buffer(casize, dataArea);

	// Build a flow object
	CclFlow sflow(Ccl::sync);

	// Clear the previous CICS error message text
	cics_message.Empty();
	cics_abcode.Empty();

	try {	// Catch block for CclClass exceptions
		connection.link(sflow, tran, &buffer);
    }    // Catch CclClass exceptions
	catch ( CclException& exception ) 
	{
		// Trace point for debugging
		ISAPITRACE("CICS ECI error - %s\n",exception.exCodeText);

		RC = exception.exCode();
		cics_message = exception.exCodeText();

		// Check for CICS Transaction abend error
		// Is it a CICS transaction abend?
		if (RC == Ccl::transaction)
		{
			// Trace point for debugging
			ISAPITRACE("CICS transaction abend - %s\n", cics_abcode);
		}
	} // end catch

	return RC;
}

////////////////////////////////////////////////////////////////
// Routine to return CICS ECI error messages based on ECI
// return codes
////////////////////////////////////////////////////////////////

const char* CEcisubs::getECImessage(int rc)

{
	char *msgptr;

        switch (rc)
         {
          case ECI_ERR_INVALID_DATA_LENGTH:
            msgptr = &"Invalid Data Length";
            break;

          case ECI_ERR_INVALID_EXTEND_MODE:
            msgptr = &"Invalid Extend Mode";
            break;

          case ECI_ERR_NO_CICS:
            msgptr = &"CICS is not running";
            break;

          case ECI_ERR_CICS_DIED:
            msgptr = &"CICS terminated unexpectedly";
            break;

          case ECI_ERR_REQUEST_TIMEOUT:
            msgptr = &"Request timed out";
            break;

          case ECI_ERR_RESPONSE_TIMEOUT:
            msgptr = &"Response timed out";
            break;

          case ECI_ERR_TRANSACTION_ABEND:
            msgptr = &"Transaction abended";
            break;

          case ECI_ERR_EXEC_NOT_RESIDENT:
            msgptr = &"Not resident";
            break;

          case ECI_ERR_SYSTEM_ERROR:
            msgptr = &"System error";
            break;

          case ECI_ERR_NULL_WIN_HANDLE:
            msgptr = &"Window handle cannot be null";
            break;

          case ECI_ERR_NULL_MESSAGE_ID:
            msgptr = &"Invalid message id";
            break;

          case ECI_ERR_THREAD_CREATE_ERROR:
            msgptr = &"Unable to create thread";
            break;

          case ECI_ERR_INVALID_CALL_TYPE:
            msgptr = &"Unknown call type";
            break;

          case ECI_ERR_ALREADY_ACTIVE:
            msgptr = &"Another request is already active";
            break;

          case ECI_ERR_RESOURCE_SHORTAGE:
            msgptr = &"A resource shortage occured";
            break;

          case ECI_ERR_NO_SESSIONS:
            msgptr = &"No sessions are available";
            break;

          case ECI_ERR_NULL_SEM_HANDLE:
            msgptr = &"The semaphore handle cannot be null";
            break;

          case ECI_ERR_INVALID_DATA_AREA:
            msgptr = &"Invalid data area pointer";
            break;

          case ECI_ERR_INVALID_VERSION:
            msgptr = &"Invalid version number";
            break;

          case ECI_ERR_UNKNOWN_SERVER:
            msgptr = &"Cannot find the named CICS server";
            break;

          case ECI_ERR_CALL_FROM_CALLBACK:
            msgptr = &"Callback routine cannot use ECI calls";
            break;

          case ECI_ERR_INVALID_TRANSID:
            msgptr = &"Transaction not found on CICS server";
            break;

          case ECI_ERR_MORE_SYSTEMS:
            msgptr = &"More systems error";
            break;

          case ECI_ERR_NO_SYSTEMS:
            msgptr = &"No systems are available";
            break;

          case ECI_ERR_SECURITY_ERROR:
            msgptr = &"Security error";
            break;

          case ECI_ERR_MAX_SYSTEMS:
            msgptr = &"Max systems exceeded";
            break;

          case ECI_ERR_MAX_SESSIONS:
            msgptr = &"Max sessions exceeded";
            break;

          case ECI_ERR_ROLLEDBACK:
            msgptr = &"CICS transaction was rolled back unexpectedly";
            break;

          default:
            msgptr = &"Unknown error";
            break;
         }

	return msgptr;
}

////////////////////////////////////////////////////////////////
// Routine to set a standard COMMAREA header fields
////////////////////////////////////////////////////////////////

void CEcisubs::setStandardHeader(pCOMMHEAD ca, 
								 long calen, 
								 int maplen, 
								 char * mapname, 
								 char * tranname,
 								 CString& cics_client_name)

{
	CString map = "";
	CString tran = "";

	// Trace point for debugging
	ISAPITRACE("Entering setStandardHeader\n");

	// Make sure that we have a COMMAREA pointer
	ISAPIASSERT(ca != NULL);

	// Get mapname and tranname into temporary CStrings
	if (mapname != NULL)
	{
		map = mapname;
	}

	if (tranname != NULL)
	{
		tran = tranname;
	}

	// set the version number to zero
	memset(ca->VERSION, '0', sizeof(ca->VERSION));

	// Set the Map Name field in the COMMAREA
	CComsubs::to_comm(map, ca->MAPNAME, sizeof(ca->MAPNAME), ALPHA_FIELD);

	// Set the User Transaction Name field in the COMMAREA
	CComsubs::to_comm(tran, ca->USRTRNID, sizeof(ca->USRTRNID), ALPHA_FIELD);

	// Set the Client Name field in the COMMAREA
	CComsubs::to_comm(cics_client_name, ca->USRTRMID, sizeof(ca->USRTRMID), ALPHA_FIELD);

	// set the user commarea length 
	CComsubs::int_to_comm(calen, ca->USRCALEN, sizeof(ca->USRCALEN));

	// set the BMS data length 
	CComsubs::int_to_comm(maplen, ca->USRMAPLN,sizeof(ca->USRMAPLN));

	// Initialize the return code and next transaction id
	memset(ca->RETCODE, '0', sizeof(ca->RETCODE));
	memset(ca->NXTRNID, ' ', sizeof(ca->NXTRNID));

	// Set default aid byte to enter key
	ca->USRAID = '\x027';

	// Set default cursor position to 0,0
	memset(ca->USRCPOS, '0', sizeof(ca->USRCPOS));
}

////////////////////////////////////////////////////////////////
// Routine to dump the contents of the standard header in a 
//  CICS COMMAREA
////////////////////////////////////////////////////////////////

void CEcisubs::dumpStandardHeader(pCOMMHEAD ca, CDumpContext & dc)

{
	// Work Variables
	CString vers;
	CString	map;
	CString mapset;
	CString clname;
	CString trans;
	CString calen;
	CString maplen;
	CString ret;
	CString nxtran;
	CString cpos;

	// Extract the data from the COMMAREA header
	CComsubs::from_comm(vers, ca->VERSION, sizeof(ca->VERSION));
	CComsubs::from_comm(map, ca->MAPNAME, sizeof(ca->MAPNAME));
	CComsubs::from_comm(mapset, ca->MAPSET, sizeof(ca->MAPSET));
	CComsubs::from_comm(clname, ca->USRTRMID, sizeof(ca->USRTRMID));
	CComsubs::from_comm(trans, ca->USRTRNID, sizeof(ca->USRTRNID));
	CComsubs::from_comm(calen, ca->USRCALEN, sizeof(ca->USRCALEN));
	CComsubs::from_comm(maplen, ca->USRMAPLN, sizeof(ca->USRMAPLN));
	CComsubs::from_comm(ret, ca->RETCODE, sizeof(ca->RETCODE));
	CComsubs::from_comm(nxtran, ca->NXTRNID, sizeof(ca->NXTRNID));
	CComsubs::from_comm(cpos, ca->USRCPOS, sizeof(ca->USRCPOS));

	// Dump the data
	dc << "VERSION      " << vers << "\n";
	dc << "MAP NAME     " << map << "\n";
	dc << "MAP SET      " << mapset << "\n";
	dc << "CLIENT NAME  " << clname << "\n";
	dc << "TRANS CODE   " << trans << "\n";
	dc << "COMMAREA LEN " << calen << "\n";
	dc << "MAP DATA LEN " << maplen << "\n";
	dc << "RETURN CODE  " << ret << "\n";
	dc << "NEXT TRANS   " << nxtran << "\n";
	dc << "CURSOR POS   " << cpos << "\n";
}
