/**************************************************************/
/*                                                            */
/*                                                            */
/*  Written by Jim MacNair - October, 1997                    */
/*                                                            */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required includes for C functions                          */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/**************************************************************/
/* Required includes for C functions                          */
/**************************************************************/

#include "comsubs.h"

/**************************************************************/
/* Routine to save an integer as a number string              */
/**************************************************************/

void int_to_comm(const long i, char *result, const int len)

{
   char temp[18];
   int maxlen;

   itoa(i, temp, 10);

   maxlen = sizeof(temp) - 1;
   if (len < maxlen)
   {
	   maxlen = len;
   } /* endif */

   if (maxlen > 0)
   {
      memset(result, '0', maxlen);
      if (strlen(temp) > 0)
      {
         memcpy(result + maxlen - strlen(temp), temp, strlen(temp));
      } /* endif */
   } /* endif */
}

/**************************************************************/
/* Routine to save a short as a number string                 */
/**************************************************************/

void short_to_comm(const short s, char *result, const int len)

{
   int i;

   i = s;
   int_to_comm(i, result, len);
}

/**************************************************************/
/* Routine to reverse the bytes in a short variable           */
/**************************************************************/

short reverseBytes(short *var)

{
    return ((*var) >> 8) + (((*var) & 255) << 8);
}

/**************************************************************/
/* Routine to reverse the bytes in an int variable            */
/**************************************************************/

int reverseBytes4(int var)

{
    return (var >> 24) + ((var & 0x00ff0000) >> 8) +
           ((var & 0x0000ff00) << 8) + ((var & 255) <<24);
}

/**************************************************************/
/* Routine to take data out of the CICS commarea and turn     */
/*  it into a valid C string, stripping trailing blanks.      */
/**************************************************************/

void char_from_comm(char *VarData, const char *commarea, const long Varlen)

{
	int  i;

	/* initialize the data area */
	memset(VarData, '\0', Varlen + 1);

	/* move the data from the COMMAREA to a variable area */
	memcpy(VarData, commarea, Varlen);

	/* strip trailing blanks */
	i = Varlen - 1;
	while (VarData[i] == ' ')
	{
		VarData[i] = '\0';

		i -= 1;
		if (i < 0)
			break;
	} /* endwhile */
}

/**************************************************************/
/* Routine to take data out of the CICS commarea and turn     */
/*  it into a valid C string, stripping trailing blanks.      */
/*  The VarData pointer is assumed to be the data area        */
/*  of a variable table entry.                                */
/*  The variable data area should be no longer than the       */
/*  maximum number of characters allowed for variable data.   */
/**************************************************************/

void from_comm(char *VarData, const char *commarea, const long Varlen)

{
	int	len;

	/* make sure we don't exceed the length of the data area */
	len = Varlen;
	if (len > MAX_DATA_LEN)
	{
		len = MAX_DATA_LEN;
	}

	/* move the data and strip trailing blanks */
	char_from_comm(VarData, commarea, len);
}

/**************************************************************/
/* Routine to take character data out of the CICS commarea    */
/* and return it into an integer.                             */
/**************************************************************/

int comm_to_int(char *commarea, long Varlen)

{
	char s[255];

	/* Extract the number */
	from_comm(s, commarea, Varlen);

	/* Convert to an integer and return the integer value  */
	return atoi(s);
}

/**************************************************************/
/* Routine to move a string into a CICS COMMAREA character    */
/*  field.                                                    */
/**************************************************************/

void char_to_comm(const char *VarData, char *CommField, int maxLen, int isNumber)

{
    int    i;

    i = strlen(VarData);

    if (i > maxLen)
     {
       i = maxLen;
     }

    /* set the field to blanks or zeros */
    if (isNumber == NUMERIC_FIELD)
    {
       /* numeric field - initialize to zeros */
       memset(CommField,'0',maxLen);

       /* copy the data, if there is any, right justified */
       if (i > 0)
       {
          memcpy(CommField + maxLen - i, VarData, i);
       }
    }
    else
    {
       /* alpha field - initialize to blanks */
       if (isNumber == ALPHA_FIELD)
       {
          memset(CommField,' ',maxLen);
       }
       else
       {
          memset(CommField,'\0',maxLen);
       }

       /* copy the data, if there is any, left justified */
       if (i > 0)
       {
          memcpy(CommField, VarData, i);
       }
    }
}

/**************************************************************/
/* Routine to look up the value of a variable and then        */
/*  copy it into the CICS commarea.                           */
/**************************************************************/

void to_comm(const char *VarName,
             char *CommField,
             VARSTRUCT *pVars,
             int maxLen,
             int isNumber)

{
    const char *VarData;

    VarData = GetVarPtr(VarName, pVars);
	char_to_comm(VarData, CommField, maxLen, isNumber);
}

void toBMSfield(VARSTRUCT *pVars,
                char      *name,
                char      *BMSfield,
                short     *BMSlen,
                short     *MAPlen,
                int       maxLen,
                int       isNumber)

{
   char   newVar[MAX_DATA_LEN + 1];
   int    i;

   to_comm(name, BMSfield, pVars, maxLen, isNumber);

   /* get the old and new values */
   GetVarVal(name, newVar, pVars);

   /* get the length of the new data */
   i = strlen(newVar);

   /* make sure it is not too long */
   if (i > maxLen)
   {
      i = maxLen;
   }

   /* set the BMS map length field */
   /* note that the BMSlen field is defined as comp-5 on the */
   /* NT workstation.  If it was a host, we would need to    */
   /* reverse the bytes, as we do for MAPlen                 */
   *BMSlen = i;

   /* set the total BMS bytes in map field                   */
   *MAPlen += i;
}

void toBMSifdiff(VARSTRUCT *pVars,
                 char      *name,
                 char      *oldname,
                 char      *BMSfield,
                 short     *BMSlen,
                 short     *MAPlen,
                 int       maxLen,
                 int       isNumber)

{
   char   newVar[MAX_DATA_LEN + 1];
   char   oldVar[MAX_DATA_LEN + 1];

   /* get the old and new values */
   GetVarVal(name, newVar, pVars);
   GetVarVal(oldname, oldVar, pVars);

   /* Are they the same?                           */
   /* if not, then we need to send in the field in */
   /* the BMS map area                             */
   if (strcmp(newVar, oldVar) != 0)
   {
      /* move the data to the BMS map field */
      toBMSfield(pVars, name, BMSfield, BMSlen, MAPlen, maxLen, isNumber);
   }
}

/**************************************************************/
/* Routine to find a debug file name in the input stream.     */
/**************************************************************/

void findDebugFile(char *input_stream, char *dbgname)

{
	char *name;
	char *name2;
	char *endname;

	dbgname[0] = '\0';

	/* see if we have a file name in the input stream */
	name = strstr(input_stream,"&INPUTDEBUGFILE");

	if (name != NULL)
	{
		/* replace the & with a string terminator */
		name[0] = '\0';

		/* move on to the next character */
		name++;

		/* find the debug file name itself */
		name2 = strstr(name, "=");

		if (name2 != NULL)
		{
			/* point at the file name */
			name2++;

			/* find the end of the file name */
			endname = strstr(name2, "&");

			/* more variables follow - terminate the file name */
			if (endname != NULL)
			{
				/* terminate the file name */
				endname[0] = '\0';
			} /* endif */

			/* move the file name */
			strcpy(dbgname,name2);

			/* lastly, append the rest of the variables */
			/* to the input_stream                      */
			if (endname != NULL)
			{
				/* replace the & we zapped */
				endname[0] = '&';
				strcat(input_stream,endname);
			} /* endif */
		} /* endif */
	} /* endif */
}

/**************************************************************/
/* Routine to insert a variable into the variable table       */
/**************************************************************/

void AddVar(const char *varname, char *vardata, VARSTRUCT *pVars)

{
	int i = 0;

	while ((i < MAX_VAR_COUNT) && (strlen(pVars[i].VarName) > 0) &&
			(strcmp(pVars[i].VarName, varname) != 0))
	{
		i++;
	} /* endwhile */

	strcpy(pVars[i].VarName,varname);
	strcpy(pVars[i].VarData,vardata);
}

/**************************************************************/
/* Routine to look up a variable name and return a pointer to */
/*  the corresponding value                                   */
/**************************************************************/

const char * GetVarPtr(const char *varname, VARSTRUCT *pVars)

{
	int i = 0;
	char *ptr = NULL;

	/* Search for the variable name */
	while ((i < MAX_VAR_COUNT) && (strlen(pVars[i].VarName) != 0) &&
			strcmp(pVars[i].VarName,varname))
	{
		i++;
	} /* endwhile */

	
	/* check if we found it */
	if (strlen(pVars[i].VarData) > 0)
	{
		ptr = pVars[i].VarData;
	}

	/* return either a pointer to the variable or NULL if not found */
	return ptr;
}

/**************************************************************/
/* Routine to update a variable in the variable table         */
/**************************************************************/

void UpdateVar(const char *varname, char *vardata, VARSTRUCT *pVars)

{
	const char *ptr = GetVarPtr(varname, pVars);

	/* check if we found the variable */
	if (ptr == NULL)
	{
		/* add it if we didn't find it */
		AddVar(varname, vardata, pVars);
	}
	else
	{
		/* update the data */
		strcpy((char *)ptr, vardata);
	}
}

/**************************************************************/
/* Routine to look up a variable name and return the          */
/*  corresponding value                                       */
/**************************************************************/

void GetVarVal(const char * varname, char * vardata, VARSTRUCT *pVars)

{
	const char *ptr = GetVarPtr(varname, pVars);

	/* check if we found the variable */
	if (ptr != NULL)
	{
		/* copy the data to the variable */
		strcpy(vardata,ptr);
	}
	else
	{
		/* Indicate that the variable was not found */
		vardata[0] = '\0';
	}
}

/**************************************************************/
/* Routine to translate strings to upper case.                */
/**************************************************************/

void strupper(char *cStr)

{
	int i = 0;

	while(cStr[i])
	{
		if ((cStr[i] >= 'a') & (cStr[i] <= 'z'))
		{
			cStr[i] = cStr[i] - 32;
		}

		i++;
	}
}

/**************************************************************/
/* Routine to convert any cOld characters in string cStr      */
/*  to cNew characters.                                       */
/*                                                            */
/* This routine is used to replace "+" characters and         */
/*  new line (\n) characters with blanks.                     */
/**************************************************************/

void strcvrt(char *cStr, char cOld, char cNew)

{
	int i = 0;

	/* Look for a string delimiter */
	while(cStr[i])
	{
		if(cStr[i] == cOld)
			cStr[i] = cNew;

		i++;
	}
}

/**************************************************************/
/* Routine to take two hex characters and turn them into      */
/*  an integer value.                                         */
/**************************************************************/

static int TwoHex2Int(char *pC)

{
	int Hi, Lo, Result;

	Hi = pC[0];
	if('0' <= Hi && Hi <= '9')
		Hi -= '0';
	else if('a' <= Hi && Hi <= 'f')
			Hi -= ('a' - 10);
		else if('A' <= Hi && Hi <= 'F')
				Hi -= ('A' - 10);

	Lo = pC[1];
	if('0' <= Lo && Lo <= '9')
		Lo -= '0';
	else if('a' <= Lo && Lo <= 'f')
			Lo -= ('a' - 10);
		else if('A' <= Lo && Lo <= 'F')
				Lo -= ('A' - 10);

	Result = Lo + 16 * Hi;
	return(Result);
}

/**************************************************************/
/* Routine to expand the escape sequences used in URL         */
/*  encoding.                                                 */
/**************************************************************/

void urlDecode(char *p)

{
	char *pD = p;

	/* Move through the string until we find a string terminator */
	while(*p)
	{
		/* Check for escape sequence character */
		if (*p == '%')  /* Escape: next 2 chars are hex    */
		{               /* representation of the actual character.*/
			/* Skip the % character */
			p++;

			/* Make sure that the two numbers are both digits */
			if(isxdigit(p[0]) && isxdigit(p[1]))
			{
				/* Convert the digits to a character */
				*pD++ = (char)TwoHex2Int(p);
				/* Move past the two digits */
				p += 2;
			}
		}
		else
		{
			/* Move the input character to the proper location */
			*pD++ = *p++;
		}
	}

	/* Terminate the string properly */
	*pD = '\0';
}

/**************************************************************/
/*                                                            */
/* Routine to replace a character with another character      */
/*  or characters.  This is used primarily to insert escape   */
/*  sequences into output data streams.                       */
/*                                                            */
/**************************************************************/

void ExpandChar(char *VarData, char ch, char *NewValue)

{
	char temp_line[MAX_LINE_LEN];

	VarData = strchr(VarData, ch);

	while (VarData != 0)
    {
		/* Save the end of the line */
		strcpy(temp_line, VarData + 1);

		/* Copy in the substitute value */
		strcpy(VarData, NewValue);

		/* Move pointer past character substitution */
		VarData = VarData + strlen(NewValue);

		/* Replace the end of the line */
		strcat(VarData, temp_line);

		/* Search for more characters */
		VarData = strchr(VarData, ch);
	} /* endwhile */
}

/**************************************************************/
/* Routine to parse the input CGI variables                   */
/**************************************************************/

void parse_input(char *varname, VARSTRUCT *pVars)

{
	char    *vardata;

	/* Get the variable name                                  */
	vardata = strchr(varname, '=');

	if (vardata != NULL)
	{
		varname[vardata - varname] = '\0';
		vardata = vardata + 1;

		/* Convert '+' and CR to blanks                         */
		strcvrt(vardata, '+', ' ');
		strcvrt(vardata, '\n', ' ');
		vardata[strlen(vardata)] = '\0';

		/* Translate name to upper case                         */
		strupper(varname);

		/* Save the variable in the variable array              */
		AddVar(varname, vardata, pVars);
	}
}

/**************************************************************/
/* Routine to read CGI variables from STDIN or QUERY_STRING   */
/**************************************************************/

void get_input_string(char *input_stream)

{
    char        *inpt_len;
    char        *inpt_type;
	char		*get_data;
    int         ilen;
    int         i;
    int         x;

/**************************************************************/
/* Check the input type                                       */
/**************************************************************/

    inpt_type = getenv("REQUEST_METHOD");
	if (inpt_type == NULL)
	{
		build_header();
		printf("<h1>Error - REQUEST_METHOD variable not set</h1><p>\n");
		build_trailer();
		exit(1);
	}

	if ((strcmp(inpt_type,"POST") != 0) &&
		(strcmp(inpt_type,"GET")  != 0))
	{
		build_header();
		printf("<h1>Error - Not POST or GET method to CGI</h1>\n");
		printf("<p><h1> %s </h1><p>",inpt_type);
		build_trailer();
		exit(1);
	}

	
	if (strcmp(inpt_type, "GET") == 0)
	{
/**************************************************************/
/* Handle form input using GET method                         */
/**************************************************************/

		/* initialize the input_stream to zero length */
		input_stream[0] = '\0';

		/* get a pointer to the data */
		get_data = getenv("QUERY_STRING");

		/* Make sure we found the variable */
		if (get_data != NULL)
		{
			strcpy(input_stream, get_data);
		}
	}
	else
	{
/**************************************************************/
/* Handle form input using POST method                        */
/**************************************************************/

/**************************************************************/
/* Get the input length                                       */
/**************************************************************/

		inpt_len = getenv("CONTENT_LENGTH");
		if (inpt_len == NULL)
		{
			build_header();
			printf("<h1>Error - CONTENT_LENGTH variable not set</h1>\n");
			build_trailer();
			exit(1);
		}

		/* Convert length from character to integer               */
		ilen = atoi(inpt_len);

		/* Error if not input                                     */
		if (ilen == 0)
		{
			build_header();
			printf("<h1>Error - Zero length to CGI</h1>\n");
			build_trailer();
			exit(1);
		}

		/* Be sure not to overflow input buffer                   */
		if (ilen > MAX_INPUT_CHAR -1)
		{
			ilen = MAX_INPUT_CHAR -1;
		}

/**************************************************************/
/* Get the input                                              */
/**************************************************************/

		/* Read the characters from stdin into input_stream       */
		i = 0;
		while (i < ilen)
		{
			/* Read a character from stdin                          */
			x = fgetc(stdin);
			if (x == EOF)
			{
				break;
			}

			/* Save character in input_stream                       */
			input_stream[i++] = x;
		}

		input_stream[i] = '\0';
	}/* end-if inpt_type == "POST" */
}

/**************************************************************/
/*                                                            */
/* Routine to build the HTML page                             */
/*                                                            */
/*  This routine will read a file and write it to stdout,     */
/*  substituting variable values for any variable names       */
/*  found which are enclosed between '&' and ';' characters.  */
/*                                                            */
/**************************************************************/

void write_html_page(VARSTRUCT *pVars, char *filename)

{
    FILE *htmlfile;
    char *pct1;
    char *pct2;
    char HTML_Line[MAX_LINE_LEN];
    char temp_line[MAX_LINE_LEN];

    if ((htmlfile = fopen(filename, "r")) == NULL)
     {
		/* Report the error to the user */
		build_header();
		printf("<h2>Error Opening HTML File - %s</h2>", filename);
		build_trailer();

		/* Exit the program */
		exit(100);
     }

    while (fgets(HTML_Line, MAX_LINE_LEN, htmlfile) != NULL)
     {
      pct1 = HTML_Line;
      while (pct1 != 0)
       {
        pct1 = strchr(pct1, '&');
        if (pct1 != 0)
         {
          pct2 = strchr(pct1 + 1, ';');

		  /* Check for no trailing delimiter */
		  if (pct2 == 0)
		  {
			  /* If no trailing delimiter, then prevent an infinite loop */
			  pct1 = 0;
		  }

          if ((pct2 != 0) && ((pct2 - pct1) < (MAX_NAME_LEN + 1)))
           {
            /* Save the end of the line */
            strcpy(temp_line, pct2 + 1);

            /* Make the variable name a proper string */
            *pct2 = '\0';

            /* Get the variable data into the HTML line */
            GetVarVal(pct1 + 1, pct1, pVars);

            /* Check for characters which must be escaped */
            /*  Note - Ampersand check must be first */
            /*ExpandChar(pct1, '&', "&amp;");
            ExpandChar(pct1, '<', "&lt;");
            ExpandChar(pct1, '>', "&gt;");*/
            ExpandChar(pct1, '%', "&#37");
            ExpandChar(pct1, '\"', "&quot");

            /* Replace the end of the line */
            strcat(pct1, temp_line);
           } /* endif */
         } /* endif */
       } /* endwhile */

		/* Ignore NULL lines */
		if (strlen(HTML_Line) > 0)
		{
			/* Write out the HTML line from the template file */
			printf(HTML_Line);
		}
     } /* endwhile */

    fclose(htmlfile);
	fflush(NULL);
}

/**************************************************************/
/* Routine to add the HTML header                             */
/**************************************************************/

void build_header()

{
        printf("<html><body>\n");
}

/**************************************************************/
/* Routine to add the HTML trailer                            */
/**************************************************************/

void build_trailer()

{
        printf("</body></html>\n");
}

/****************/
/* End of File  */
/****************/