/**************************************************************/
/*                                                            */
/*	CGI program to verify userids and passwords.  It calls    */
/*   a CICS program called VERIFYPW, which is part of the     */
/*   CA03 supportpac, using the CICS client ECI.              */
/*                                                            */
/*  Written by Jim MacNair - January, 1998                    */
/*                                                            */
/*  This program demonstrates basic techniques for a proof    */
/*   of concept for CICS access from the Internet.            */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required include for C functions                           */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>

/**************************************************************/
/* Include for common subroutines                             */
/**************************************************************/

#include "comsubs.h"
#include "ecimsg.h"

/**************************************************************/
/* Required include for CICS ECI calls                        */
/**************************************************************/

#define CICS_W32
#include <cics_eci.h>

/**************************************************************/
/* COMMAREA definition                                        */
/**************************************************************/

#include "verifypw.h"

/**************************************************************/
/*                                                            */
/* Variable name and value array definition                   */
/*                                                            */
/* Input variables are saved in this array.  Other variables  */
/*  can be added as well.  To build the output HTML page,     */
/*  the file containing the HTML page is written to           */
/*  stdout, with variable names being substituted with the    */
/*  corresponding values.  Variables are recognized by        */
/*  looking for a double % sequence.                          */
/*                                                            */
/**************************************************************/

/* Name of the HTML prototype file */
#define HTML_LOGIN_FILE	"logon.htm"
#define HTML_INQY_FILE	"credinqy.htm"

/* CICS program name */
#define CICSPROG "VERIFYPW"

/* Default temporary state file directory location       */
/* Note that this can be overriden by using the CICSTEMP */
/* environment variable.                                 */
#define CICSTEMP "c:\\CICSTEMP"
#define MAX_CICSTEMP_LEN MAX_DATA_LEN - 31
#define FILE_PREFIX "CICS"

/* CICS Status file header structure */
typedef struct {
   char userName[8];
   char passWord[8];
   char sysName[8];
   char clientName[4];
   char replayCount[8];
   char tctuaLength[4];
   char caLength[8];
} STATUS_FILE_HEADER;

/**************************************************************/
/*                                                            */
/* Routine to generate a unique temporary file name.          */
/* This file will be used to save the CICS status on the      */
/* web server.  The CICS status includes the user id and      */
/* password, system and client names, a counter used to       */
/* detect replays and the latest CICS COMMAREA.               */
/*                                                            */
/* The file name will be located in the C:\CICSTEMP           */
/* directory, unless the CICSTEMP environment variable        */
/* is used to override this.  The file name will start        */
/* with the characters CICS, and will include the CICS        */
/* system and client names, followed by a temporary file      */
/* name (number) to make the file unique.  This file name     */
/* must be sent to the browser as a hidden variable, and      */
/* used to retrieve the file contents after the next          */
/* user input.                                                */
/*                                                            */
/* The CICSTEMP environment variable must be less than        */
/* 64 characters long.  This restriction is to prevent the    */
/* total file name from exceeding the maximum variable table  */
/* variable length of MAX_DATA_LEN.                           */
/*                                                            */
/**************************************************************/

void generateFilename(char* filename, const char* clientname, const char* systemname)

{
	/* Work variables */
	char	dirname[MAX_DATA_LEN + 1];
	char	*envvar;
	char	fileprefix[31];
	char	*tempname;

	/* initialize the work variables */
	memset(dirname, 0, sizeof(dirname));

	/* Initialize the file name */
	filename[0] = 0;

	/* Get the default directory name */
	memcpy(dirname, CICSTEMP, sizeof(CICSTEMP));

	/* check if the CICSTEMP environment variable is set */
	envvar = getenv("CICSTEMP");
	if (envvar != NULL)
	{
		/* We have one, check if it is too long */
		if (strlen(envvar) <= MAX_CICSTEMP_LEN)
		{
			/* replace the default drive and directory location */
			strcpy(envvar, dirname);
		}
	}

	/* Build the rest of the file name */

	/* Concate to the directory name                           */
	/* make sure we have a back slash character if we need one */
	if ((strlen(dirname) > 0) && (dirname[strlen(dirname)] != '\\'))
	{
		strcat(dirname, "\\");
	}
	/* build the file prefix */
	fileprefix[0] = 0;
	strcat(fileprefix, FILE_PREFIX);	/* Start name with CICS */
	strcat(fileprefix, systemname);		/* Add the system name */
	strcat(fileprefix, clientname);		/* Add the client name */

	/* Now, generate the temporary file name */
	tempname = _tempnam(dirname, fileprefix);

	/* Check if it worked */
	if (tempname != NULL)
	{
		/* Copy the file name to the return area */
		strcpy(filename, tempname);

		/* Free the temporary file name area generated by _tempnam */
		free(tempname);
	}
}

/**************************************************************/
/*                                                            */
/* This routine will create the current CICS status file.     */
/* The status file name is generated when a user              */
/* signs on, and is saved in all subsequent                   */
/* web pages as a hidden variable.  It could also be saved    */
/* as a cookie.  The replay count is used to detect           */
/* replays of a previous HTML page.  It will be automatically */
/* updated before the temporary file is written.              */
/*                                                            */
/**************************************************************/

int	saveTempfileData(char* filename, 
					 char* user, 
					 char* pw, 
					 char* client, 
					 char* sys, 
					 int replay,
					 const int tctualen,
					 const char* tctua)

{
	int		rc=0;		/* return code variable */
	FILE	*tempfile;	/* file handle for the temporary file */
	STATUS_FILE_HEADER	hdrData;	/* Status file header data */

	/* open the file for output, erasing any previous files */
	tempfile = fopen(filename, "wb");

	/* Check if it worked */
	if (tempfile == NULL)
	{
		/* Didn't work, set return code to 1 */
		rc = 1;
	}
	else
	{
		/* file is open, build the first part of the file      */
		/* Get the user id, padded with spaces to 8 characters */
		char_to_comm(user, hdrData.userName, sizeof(hdrData.userName), ALPHA_FIELD); 

		/* Get the password, padded with spaces to 8 characters */
		char_to_comm(pw, hdrData.passWord, sizeof(hdrData.passWord), ALPHA_FIELD); 

		/* Get the system name, padded with spaces to 8 characters */
		char_to_comm(sys, hdrData.sysName, sizeof(hdrData.sysName), ALPHA_FIELD); 

		/* Get the client name, padded with spaces to 4 characters */
		char_to_comm(client, hdrData.clientName, sizeof(hdrData.clientName), ALPHA_FIELD); 

		/* Set the replay count to zero */
		int_to_comm(replay, hdrData.replayCount, sizeof(hdrData.replayCount));

		/* Set the TCT user area length field */
		int_to_comm(tctualen, hdrData.tctuaLength, sizeof(hdrData.tctuaLength));

		/* Set the COMMAREA length field to 0 */
		int_to_comm(0, hdrData.caLength, sizeof(hdrData.caLength));

		/* Write the system name to the file */
		fwrite(&hdrData, 1, sizeof(hdrData), tempfile);

		/* Check if there is a TCTUA to save */
		if (tctualen > 0)
		{
			fwrite(tctua, 1, tctualen, tempfile);
		}

		// Close the file */
		fclose(tempfile);
	}

	return rc;
}

/**************************************************************/
/* Routine to identify a variable and copy the data           */
/*  into the commarea.                                        */
/**************************************************************/

void build_commarea(VARSTRUCT *pVars,
                    PVERIFYCOMM pCommArea)

{
	/* Set the version number */
	memcpy(pCommArea->VERSION,"00",sizeof(pCommArea->VERSION));

	/* Fill in the userid and password */
    to_comm("CICSUSER", pCommArea->CICSUSER,
            pVars, sizeof(pCommArea->CICSUSER),
            ALPHA_FIELD);

    to_comm("CICSPW", pCommArea->CICSPW,
            pVars, sizeof(pCommArea->CICSPW),
            ALPHA_FIELD);

    to_comm("CICSCLIENT", pCommArea->CLIENTNAME,
            pVars, sizeof(pCommArea->CLIENTNAME),
            ALPHA_FIELD);
}

int main(int argc, char *argv[])

{
/**************************************************************/
/* Input variables from form                                  */
/**************************************************************/

    VERIFYCOMM  *pCommArea;
    VARSTRUCT   *pVars;
    char        input_stream[MAX_INPUT_CHAR];
    char        *token;
    short       rc;
    char        cics_user[] = "USERID  ";
    char        cics_pw[]   = "PASSWORD";
	char        cics_sys[9];
	char		cics_client[5];
	char		abendcode[ABCODE_LEN+1];
    char        *dbginputname;             /* debug file name */
    char        dbgname[MAX_DATA_LEN + 1]; /* debug file name */
    char        *filename;
	char		statusfile[MAX_DATA_LEN + 1];
    FILE        *dbgfile;                  /* debug file */

/**************************************************************/
/*     Turn off buffering for stdout                          */
/**************************************************************/
    setvbuf(stdout,NULL,_IONBF,0);

/**************************************************************/
/* Start a new html page                                      */
/**************************************************************/
    printf("Content-type: text/html\n\n");
    fflush(NULL);

/**************************************************************/
/* Allocate a commarea and variable area                      */
/**************************************************************/
    pCommArea = malloc(sizeof(VERIFYCOMM));
    pVars = (VARSTRUCT *)malloc(sizeof(VARSTRUCT) * (MAX_VAR_COUNT + 1));
    if ((pCommArea == NULL) || (pVars == NULL))
      {
        build_header();
        printf("<h1>Error allocating storage</h1>");
        build_trailer();
        exit(1);
      }

/**************************************************************/
/* Initialize the commarea variables to default values        */
/**************************************************************/

    memset(pCommArea,'\0',sizeof(VERIFYCOMM));
    memset(pVars,'\0',(sizeof(VARSTRUCT) * (MAX_VAR_COUNT + 1)));
	memset (cics_sys, '\0', sizeof(cics_sys));

/**************************************************************/
/* Get the form input into variable input_stream              */
/**************************************************************/

    if (argc>1)        /* check for debugging */
    {
       dbginputname = argv[1];
       if (strcmp(dbginputname,"debug") == 0)
       {
          strcpy(input_stream,"&CICSUSER=USERID&CICSPW=PASSWORD&CICSSYSTEM=");
          strcat(input_stream,"&CICSCLIENT=JIMM&SUBMIT=Logon to CICS");
       }
       else
       {
          if ((dbgfile = fopen(dbginputname,"r")) != NULL)
          {
             fgets(input_stream, MAX_INPUT_CHAR, dbgfile);
             fclose(dbgfile);
          } /* endif */
       } /* endif */
    }
    else
    {
       get_input_string(input_stream);
    } /* endif */

/**************************************************************/
/* Check if we want to write out the input for debuging       */
/* purposes.  To easily capture the input form variables,     */
/* set a variable named INPUTDEBUGFILE to the name of         */
/* the file to be written out with the input from the         */
/* HTML form.  This input can then be fed back into           */
/* the program as a test case.  This allows easy debugging    */
/* of the CGI program outside of a web server.                */
/**************************************************************/

    /* find the debug file name in the input stream */
    /* and remove it from the input stream.         */
    findDebugFile(input_stream, dbgname);
    if (strlen(dbgname) != 0)
    {
       if ((dbgfile = fopen(dbgname, "w")) != NULL)
       {
          fprintf(dbgfile,"%s",input_stream);
          fclose(dbgfile);
       } /* endif */
    } /* endif */

/**************************************************************/
/* Parse the input into fields                                */
/**************************************************************/

    /* get the first variable                                 */
    token = strtok(input_stream, "&");
    while( token != NULL )
     {
      /* Expand escape sequences to proper characters         */
      urlDecode(token);

      /* Process the variable                                 */
      parse_input(token, pVars);

      /* Get the next variable                                */
      token = strtok(NULL, "&");
     }

/**************************************************************/
/* Build the CommArea                                         */
/**************************************************************/

    build_commarea(pVars, pCommArea);

/**************************************************************/
/* Extract the CICS SYSTEM, if specified                      */
/**************************************************************/

    to_comm("CICSSYS", cics_sys, pVars, sizeof(cics_sys), CICS_FIELD);

/**************************************************************/
/* Now issue the ECI call                                     */
/**************************************************************/

	rc = performECI(CICSPROG, cics_user, cics_pw, cics_sys,
					(char *) pCommArea, sizeof(VERIFYCOMM), pVars, abendcode);

    filename = HTML_LOGIN_FILE;

/**************************************************************/
/* Check the results                                          */
/**************************************************************/

    if (rc == ECI_NO_ERROR)
    {
	   /* Check for a normal return code */
       if (memcmp(pCommArea->RESP, "00000", 5) == 0)
       {
			/* display the initial inquiry form */
			filename = HTML_INQY_FILE;

			/* check if the CICS Client name was changed */
			char_from_comm(cics_client, pCommArea->CLIENTNAME, sizeof(pCommArea->CLIENTNAME));

			/* Create a temporary status file */
			/* First, create a temporary file name */
			generateFilename(statusfile, cics_client, cics_sys);

			/* Save the initial transaction status.            */
			/* There is no TCTUA, so pass zeros for the length */
			/* and NULL for the address.                       */
			saveTempfileData(statusfile, cics_user, cics_pw, cics_client, cics_sys, 0, 0, NULL);

			/* Add the file name and replay count to the variable table */
			AddVar("CICSSTATUSFILE", statusfile, pVars);
			AddVar("REPLAYCOUNT", "00000000", pVars);
       }
       else
       {
		    /* Set a variable with the error message */
		    AddVar("ERRMSG", "Invalid Userid or Password", pVars);
       } /* endif */
    }
	else
	{
		/* Check for invalid user id and password */
		/* If the mirror program detects an  invalid user id and/or */
		/* password, it will issue an AEY7 abend */
		if ((rc == ECI_ERR_TRANSACTION_ABEND) && (strcmp(abendcode, "AEY7") == 0))
		{
		    /* Set a variable with the error message */
		    AddVar("ERRMSG", "Invalid Userid or Password", pVars);
		}
		else
		{
			/* Create a user error message */
			UpdateVar("ERRMSG", "ERROR CALLING CICS", pVars);
		}
	}

/**************************************************************/
/* Build the new HTML page                                    */
/**************************************************************/

    write_html_page(pVars, filename);

/**************************************************************/
/* Finished - return normally                                 */
/**************************************************************/

    return(0);
}

/****************/
/* End of File  */
/****************/